#include "QueryWidget.h"

#include <QWidget>
#include <QTabWidget>
#include <QPushButton>
#include <QCheckBox>
#include <QHBoxLayout>
#include <QVBoxLayout>
#include <QGroupBox>

#include <ArmarXCore/core/exceptions/local/ExpressionException.h>


namespace armarx::armem::gui
{

    QueryWidget::QueryWidget()
    {
        QHBoxLayout* hlayout1 = new QHBoxLayout();
        QHBoxLayout* hlayout2 = new QHBoxLayout();
        QVBoxLayout* vlayout = new QVBoxLayout();

        _dataCheckBox = new QCheckBox("Get Data");
        _dataCheckBox->setChecked(true);

        _storeInLTMButton = new QPushButton("Store query result in LTM");

        //_tabWidget = new QTabWidget();

        _snapshotSelectorWidget = new SnapshotSelectorWidget();
        //_tabWidget->addTab(_snapshotSelectorWidget, QString("Snapshots"));

        hlayout1->addWidget(_dataCheckBox);
        hlayout1->addWidget(_storeInLTMButton);

        hlayout2->addWidget(_snapshotSelectorWidget);

        const int margin = 0;
        vlayout->setContentsMargins(margin, margin, margin, margin);

        vlayout->addLayout(hlayout1);
        vlayout->addLayout(hlayout2);

        // Public connections.
        connect(_storeInLTMButton, &QPushButton::pressed, this, &This::storeInLTM);

        setLayout(vlayout);
    }


    armem::DataMode QueryWidget::dataMode() const
    {
        return _dataCheckBox->isChecked()
               ? armem::DataMode::WithData
               : armem::DataMode::NoData;
    }

    armem::client::QueryInput QueryWidget::queryInput()
    {
        armem::client::query::Builder qb(dataMode());
        qb.queryTargets(_snapshotSelectorWidget->queryTargets())
        .coreSegments().all().queryTargets(_snapshotSelectorWidget->queryTargets())
        .providerSegments().all().queryTargets(_snapshotSelectorWidget->queryTargets())
        .entities().all().queryTargets(_snapshotSelectorWidget->queryTargets())
        .snapshots(_snapshotSelectorWidget->selector());

        return qb.buildQueryInput();
    }

}
