#include "SkillManagerComponentPlugin.h"

#include <ArmarXCore/core/Component.h>

namespace armarx::plugins
{
    void SkillManagerComponentPlugin::preOnInitComponent()
    {}

    void SkillManagerComponentPlugin::preOnConnectComponent()
    {}

    void SkillManagerComponentPlugin::postCreatePropertyDefinitions(PropertyDefinitionsPtr& properties)
    {}
}


namespace armarx
{
    SkillManagerComponentPluginUser::SkillManagerComponentPluginUser()
    {
        addPlugin(plugin);
    }

    void SkillManagerComponentPluginUser::addProvider(const skills::manager::dto::ProviderInfo& info, const Ice::Current&)
    {
        std::lock_guard l(skillProviderMapMutex);
        if (skillProviderMap.find(info.providerName) == skillProviderMap.end())
        {
            ARMARX_INFO << "Adding a provider with name '" << info.providerName << "'.";
            skillProviderMap.insert({info.providerName, info.provider});
        }
    }

    void SkillManagerComponentPluginUser::removeProvider(const std::string& providerName, const Ice::Current&)
    {
        std::lock_guard l(skillProviderMapMutex);
        if (auto it = skillProviderMap.find(providerName); it != skillProviderMap.end())
        {
            skillProviderMap.erase(it);
        }
    }

    skills::provider::dti::SkillProviderMap SkillManagerComponentPluginUser::getSkillProviders(const Ice::Current&)
    {
        return skillProviderMap;
    }

    void SkillManagerComponentPluginUser::executeSkill(const skills::manager::dto::SkillExecutionInfo& info, const Ice::Current&)
    {
        if (auto it = skillProviderMap.find(info.providerName); it != skillProviderMap.end())
        {
            skills::callback::dti::SkillProviderCallbackInterfacePrx myPrx;
            getProxy(myPrx, -1);

            skills::provider::dto::SkillExecutionInfo exInfo;
            exInfo.skillName = info.skillName;
            exInfo.callbackInterface = myPrx;
            exInfo.params = info.params;
            exInfo.waitUntilSkillFinished = info.waitUntilSkillFinished;

            it->second->executeSkill(exInfo);
        }
        else
        {
            throw LocalException("Could not execute a skill of provider '" + info.providerName + "' because the provider does not exist.");
        }
    }

    void SkillManagerComponentPluginUser::abortSkill(const std::string& providerName, const std::string& skillName, const Ice::Current &current)
    {
        if (auto it = skillProviderMap.find(providerName); it != skillProviderMap.end())
        {
            it->second->abortSkill(skillName, false);
        }
    }

    void SkillManagerComponentPluginUser::updateStatusForSkill(const skills::provider::dto::SkillStatusUpdate& statusUpdate, const Ice::Current&)
    {
        (void) statusUpdate;
        // If you want to use the status, implement this method!
    }
}
